@@ -1,9 +1,9 @@
 package SQLite::Counter::Simple;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2021-06-18'; # DATE
+our $DATE = '2021-06-21'; # DATE
 our $DIST = 'SQLite-Counter-Simple'; # DIST
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 use 5.010001;
 use strict;
@@ -58,11 +58,6 @@ _
 };
 
 our %argspecs_common = (
-    counter => {
-        summary => 'Counter name, defaults to "default" if not specified',
-        schema => 'str*',
-        pos => 0,
-    },
     path => {
         summary => 'Database path',
         description => <<'_',
@@ -79,6 +74,32 @@ _
     },
 );
 
+our %argspec_counter = (
+    counter => {
+        summary => 'Counter name, defaults to "default" if not specified',
+        schema => 'str*',
+        pos => 0,
+    },
+);
+
+sub _increment {
+    my ($dbh, $counter, $increment, $dry_run) = @_;
+
+    $increment //= 1;
+
+    $dbh->begin_work;
+    # XXX use prepared statement for speed
+    $dbh->do("INSERT OR IGNORE INTO counter (name,value) VALUES (?,?)", {}, $counter, 0);
+    my ($val) = $dbh->selectrow_array("SELECT value FROM counter WHERE name=?", {}, $counter);
+    return [500, "Cannot create counter '$counter' (1)"] unless defined $val;
+    $val += $increment;
+    unless ($dry_run) {
+        $dbh->do("UPDATE counter SET value=? WHERE name=?", {}, $val, $counter);
+    }
+    $dbh->commit;
+    [200, "OK", $val];
+}
+
 $SPEC{increment_sqlite_counter} = {
     v => 1.1,
     summary => 'Increment a counter in a SQLite database and return the new incremented value',
@@ -94,6 +115,7 @@ counter been incremented, but the counter will not be actually incremented.
 _
     args => {
         %argspecs_common,
+        %argspec_counter,
         increment => {
             summary => 'Specify by how many should the counter be incremented',
             schema => 'int*',
@@ -111,17 +133,34 @@ sub increment_sqlite_counter {
     my ($res, $dbh) = _init(\%args);
     return $res unless $res->[0] == 200;
 
-    $dbh->begin_work;
-    # XXX use prepared statement for speed
-    $dbh->do("INSERT OR IGNORE INTO counter (name,value) VALUES (?,?)", {}, $args{counter}, 0);
-    my ($val) = $dbh->selectrow_array("SELECT value FROM counter WHERE name=?", {}, $args{counter});
-    return [500, "Cannot create counter '$args{counter}' (1)"] unless defined $val;
-    $val += ($args{increment} // 1);
-    unless ($args{-dry_run}) {
-        $dbh->do("UPDATE counter SET value=? WHERE name=?", {}, $val, $args{counter});
+    _increment($dbh, $args{counter}, $args{increment}, $args{-dry_run});
+}
+
+$SPEC{dump_sqlite_counters} = {
+    v => 1.1,
+    summary => 'Return all the counters in a SQLite database as a hash',
+    description => <<'_',
+_
+    args => {
+        %argspecs_common,
+    },
+    features => {
+        dry_run => 1,
+    },
+};
+sub dump_sqlite_counters {
+    my %args = @_;
+
+    my ($res, $dbh) = _init(\%args);
+    return $res unless $res->[0] == 200;
+
+    my $sth = $dbh->prepare("SELECT name,value FROM counter");
+    $sth->execute;
+    my %counters;
+    while (my $row = $sth->fetchrow_hashref) {
+        $counters{$row->{name}} = $row->{value};
     }
-    $dbh->commit;
-    [200, "OK", $val];
+    [200, "OK", \%counters];
 }
 
 $SPEC{get_sqlite_counter} = {
@@ -134,6 +173,7 @@ Undef (exit code 1 in CLI) can be returned if counter does not exist.
 _
     args => {
         %argspecs_common,
+        %argspec_counter,
     },
     features => {
         dry_run => 1,
@@ -149,6 +189,44 @@ sub get_sqlite_counter {
     [200, "OK", $val, {'cmdline.exit_code'=>defined $val ? 0:1}];
 }
 
+sub new {
+    my ($class, %args) = @_;
+
+    my ($res, $dbh) = _init(\%args);
+    die "Can't initialize: $res->[0] - $res->[1]" unless $res->[0] == 200;
+
+    my $obj = {
+        path => delete $args{path},
+        dbh  => $dbh,
+    };
+
+    delete $args{counter};
+    keys(%args) and die "Unknown constructor argument(s): ".join(", ", sort keys %args);
+    bless $obj, $class;
+}
+
+sub increment {
+    my ($self, %args) = @_;
+
+    my $counter = delete($args{counter}) // 'default';
+    my $increment = delete($args{increment});
+    keys(%args) and die "Unknown argument(s): ".join(", ", sort keys %args);
+    my $res = _increment($self->{dbh}, $counter, $increment); # doesn't support dry-run
+    die "Can't increment counter '$counter': $res->[0] - $res->[1]"
+        unless $res->[0] == 200;
+    $res->[2];
+}
+
+sub get {
+    my ($self, %args) = @_;
+
+    my $counter = delete($args{counter}) // 'default';
+    keys(%args) and die "Unknown argument(s): ".join(", ", sort keys %args);
+    my ($val) = $self->{dbh}->selectrow_array("SELECT value FROM counter WHERE name=?", {}, $counter);
+    $val;
+}
+
+
 1;
 # ABSTRACT: A simple counter using SQLite
 
@@ -164,7 +242,7 @@ SQLite::Counter::Simple - A simple counter using SQLite
 
 =head1 VERSION
 
-This document describes version 0.002 of SQLite::Counter::Simple (from Perl distribution SQLite-Counter-Simple), released on 2021-06-18.
+This document describes version 0.003 of SQLite::Counter::Simple (from Perl distribution SQLite-Counter-Simple), released on 2021-06-21.
 
 =head1 SYNOPSIS
 
@@ -221,6 +299,59 @@ a single CLI script invocation.
 =head1 FUNCTIONS
 
 
+=head2 dump_sqlite_counters
+
+Usage:
+
+ dump_sqlite_counters(%args) -> [$status_code, $reason, $payload, \%result_meta]
+
+Return all the counters in a SQLite database as a hash.
+
+This function is not exported.
+
+This function supports dry-run operation.
+
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<path> => I<filename>
+
+Database path.
+
+If not specified, will default to $HOME/counter.db. If file does not exist, will
+be created by DBD::SQLite.
+
+If you want an in-memory database (that will be destroyed after your process
+exits), use C<:memory:>.
+
+
+=back
+
+Special arguments:
+
+=over 4
+
+=item * B<-dry_run> => I<bool>
+
+Pass -dry_run=E<gt>1 to enable simulation mode.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element ($status_code) is an integer containing HTTP-like status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+($reason) is a string containing error message, or something like "OK" if status is
+200. Third element ($payload) is the actual result, but usually not present when enveloped result is an error response ($status_code is not 2xx). Fourth
+element (%result_meta) is called result metadata and is optional, a hash
+that contains extra information, much like how HTTP response headers provide additional metadata.
+
+Return value:  (any)
+
+
+
 =head2 get_sqlite_counter
 
 Usage:
@@ -346,6 +477,68 @@ that contains extra information, much like how HTTP response headers provide add
 
 Return value:  (any)
 
+=head1 METHODS
+
+Aside from the functional interface, this module also provides the OO interface.
+
+=head2 new
+
+Constructor.
+
+Usage:
+
+ my $counter = SQLite::Counter::Simple->new(%args);
+
+Known arguments (C<*> marks required argument):
+
+=over
+
+=item * path
+
+Database path, defaults to C<$HOME/counter.db>.
+
+=back
+
+=head2 increment
+
+Increment counter.
+
+Usage:
+
+ my $newval = $counter->increment(%args);
+
+Arguments:
+
+=over
+
+=item * counter
+
+Counter name, defaults to C<default>.
+
+=item * increment
+
+Increment, defaults to 1.
+
+=back
+
+=head2 get
+
+Get current value of a counter.
+
+Usage:
+
+ my $val = $counter->get(%args);
+
+Arguments:
+
+=over
+
+=item * counter
+
+Counter name, defaults to C<default>.
+
+=back
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/SQLite-Counter-Simple>.
